%% Stationarity Test Function
% using TF representation, Stationary Surrogates and distance-based Statistical test. 
%
% References: see perso.ens-lyon.fr/pierre.borgnat
%
% 1) "Testing Stationarity with Time-Frequency Surrogates", 
% J. Xiao, P. Borgnat, P. Flandrin, 15th European Signal Processing Conference EUSIPCO-2007, 
%   Poznan (Pologne), 3-7 septembre 2007
% 2)  "Sur un test temps-frquence de stationnarit", J. Xiao, P. Borgnat, P. Flandrin, 
% 21e Colloque sur le Traitement du Signal et des Images. GRETSI-2007, Troyes (France), 11-14 septembre 2007.
%
%
% Input: x      signal
%       opttfr  options of the TF Representation; struct [see inside file for more options]
%           .choixtfr   Choice of the TFR ('mtfr' Multi-Taper Spectrogram is recothe only one well supported now)
%           .Mh         Number of Tapers used if 'mtfr'
%           .Nfft       Number of points in frequency for the TFR
%           .Nh         Length (in points) of the TFR window
%           .dt         Space bewteen time samples
%       opttest options of the statistical test [see inside file for more options]
%           .JJ         Number of Surrogates
%           .fa_rate    False Alarm Rate
%       optfig  options for displaying intermediate figures; 1 will display all of them.
%           [see inside file for more options]
%           
% Output:   theta1      Test Statistics for original signal
%           theta0      Test Statistics for each surrogate (vector of size opttest.JJ)
%           gam_hat     Estimation of [alpha,beta] of the Gamma distribution of the {Theta_0}
%           gam_thresh  Threshold of the test in the Theta space (based on opttest.fa_rate)
%           res_test    Result of the test: 0=stationary, 1=non-stationary
%           INS         Index of Non-stationarity
%           INS_thresh  Threshold of the test in INS space
%
% Need: phasemodul dist_locvsglob statio_test_theta tfrsp_h tfrsp_hm mean_hmt5
%
% Usage: [theta1,theta0,gam_hat,gam_thresh,INS,res_test]=statio_test_funct(x,opttfr,opttest,optfig) ;
%
% Jun Xiao, Pierre Borgnat & Patrick Flandrin 
% 09/2007


function [theta1,theta0,gam_hat,gam_thresh,res_test,INS,INS_thresh,opttfr,opttest]=statio_test_funct(x,opttfr,opttest,optfig) ;

disp('---- Begin new Stationarity test ----');

%% INPUT variable and DEFAULTS

[l,c]=size(x);
if l>1,
    x=x.';
end;
Nx=length(x);

x = x/std(x) ;

if nargin<2,
    opttfr=struct;
end;

try, opttfr.choixtfr; 
    catch opttfr.choixtfr='mtfr';   
end;
choixtfr=opttfr.choixtfr;

switch opttfr.choixtfr
    case 'mtfr'
        try, opttfr.Nh0;    catch opttfr.Nh0=0.2;               end
        Nh0=opttfr.Nh0;
        try, opttfr.Nh;     catch opttfr.Nh=round(Nx*Nh0/2)*2-1;end
        Nh=opttfr.Nh; 
        try, opttfr.Nfft;   catch opttfr.Nfft=2^ceil(log2(Nh)); end
        Nfft=opttfr.Nfft; 
        try, opttfr.dt;     catch opttfr.dt=fix((Nh+1)/8);      end
        dt=opttfr.dt;
        try, opttfr.sides;  catch opttfr.sides=(Nh+1)/2;        end
        sides=opttfr.sides;
        try, opttfr.tt;     catch opttfr.tt=sides:dt:Nx-sides;  end
        tt=opttfr.tt;
        try, opttfr.ttred;  catch opttfr.ttred=tt;              end
        ttred=opttfr.ttred;
        try, opttfr.Mh;     catch opttfr.Mh=5;                  end
        try, opttfr.tm;     catch opttfr.tm=5;                  end
        try, opttfr.opt1;   catch opttfr.opt1=1;                end  
        Mh=opttfr.Mh;   tm=opttfr.tm;   opt1=opttfr.opt1; 
        
    case 'gabor'
        try, opttfr.Q;      catch opttfr.Q=2;                   end
        Q=opttfr.Q;
        try, opttfr.Nh0;    catch opttfr.Nh0=0.1;               end
        Nh0=opttfr.Nh0;
        try, opttfr.Nh;     catch opttfr.Nh=round(Nx*Nh0/2)*2-1;end
        Nh=opttfr.Nh; 
        try, opttfr.Nfft;   catch opttfr.Nfft=2^ceil(log2(Nh)-2);       end
        Nfft=opttfr.Nfft;
        try, opttfr.Nt;     catch opttfr.Nt=Nx/Nfft*Q;                  end
        Nt=opttfr.Nt;
        try, opttfr.dt;     catch opttfr.dt=Nx/Nt;                      end
        dt=opttfr.dt;
        try, opttfr.sides;  catch opttfr.sides=ceil((Nh+1)/2/dt);       end
        sides=opttfr.sides;
        try, opttfr.tt;     catch opttfr.tt=round(((1:Nt)+0.5)/Nt*Nx);  end
        tt=opttfr.tt;
        try, opttfr.ttred;  catch opttfr.ttred=sides:dt:Nx-sides;          end
        ttred=opttfr.ttred;
        
end;
% For sake of commodity, all parameters are taken out of the general structure.

if nargin<3,
    opttest=struct;
end;

try, opttest.opt_dist;      catch opttest.opt_dist=8;       end;
try, opttest.JJ;            catch opttest.JJ=50;         end;

try, opttest.doBS;          catch opttest.doBS=0;           end;
try, opttest.fa_rate;       catch opttest.fa_rate=0.05;     end;
try, opttest.Nhist;         catch opttest.Nhist=20;         end;

opt_dist=opttest.opt_dist;
JJ=opttest.JJ;
doBS=opttest.doBS;
fa_rate=opttest.fa_rate;
Nhist=opttest.Nhist;

if nargin<4,
    optfig=struct;
end;
if ~isstruct(optfig),
    if optfig==1,
    optfig.tfrdisplay=1;
    optfig.figdisplay=1;
    optfig.resdisplay=1;
    optfig.dyn=20;
    end;
end;
    
try, optfig.tfrdisplay;     catch optfig.tfrdisplay=1;       end;
try, optfig.figdisplay;     catch optfig.figdisplay=0;       end;
try, optfig.resdisplay;     catch optfig.resdisplay=1;       end;
try, optfig.dyn;            catch optfig.dyn=20;            end;


tfrdisplay=optfig.tfrdisplay;
figdisplay=optfig.figdisplay;
resdisplay=optfig.resdisplay;
dyn=optfig.dyn;

%% TF REPRESENTATION %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

switch choixtfr,
    case 'gabor';
%%% with Gabor TFR
    disp('-- with Gabor TFR');
    disp(['Nt = ' num2str(Nt) ' / Nfft = ' num2str(Nfft) ' / Q = ' num2str(Q)]);
    disp(['Nh = ' num2str(Nh) ' / sides = ' num2str(sides)]);
    [tfrx,dgrx]=tfrgabor(hilbert(x),Nt,Q,tftb_window(Nh,'hanning'),1) ;
    case 'mtfr';
%%% with Multi-Taper spectrogram
    disp(['-- with Multi-Taper spectrogram']);
    disp(['Nx = ' num2str(Nx) ' / Nfft = ' num2str(Nfft) ' / Nh = ' num2str(Nh)]);
    disp(['Mh = ' num2str(Mh) ' / tm = ' num2str(tm) ' / opt1 = ' num2str(opt1)]);
    MSp = tfrsp_hm(hilbert(x).',tt,Nfft,Nh,Mh,tm) ; % computation of MultTaper Spectrogram
    tfrx = mean_hmt5(MSp,opt1) ;                    % means over tapers
end;

if tfrdisplay,
    figure(2); clf; imageTF(tfrx(1:Nfft/2,:),dyn,1,0); axis('square'); title('Original Signal TFR')
end;

%% %---- STATISTICS for the test --------------------------------------
tfr=tfrx(1:Nfft/2,:);
[theta1,Cn_dist,Cn_mean]=statio_test_theta(tfr,ttred,opt_dist);

if figdisplay,
    figure(4); clf;
    plot((Cn_dist-Cn_mean).^2,'mo-','LineWidth',2); hold on
    title('cn vs. time for Signal (magenta) and Surrogates (blue)');
end;
    
    % tfr_margF=mean(tfr,2);

%% COMPUTATION of SURROGATES %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% JJ=50;

theta0=zeros(1,JJ);
for jj=1:JJ,
disprog(jj,JJ,20);

%---- SURROGATE Data Creation + TFR ---------------------------

z = phasemodul(x,Nx);
switch choixtfr,
case 'gabor';
%%% with Gabor TFR
    [tfrz,dgrz]=tfrgabor(hilbert(z),Nt,Q,tftb_window(Nh,'hanning'),0) ;
case 'mtfr';
%%% with Multi-Taper spectrogram
    MSp = tfrsp_hm(hilbert(z),tt,Nfft,Nh,Mh,tm) ; % computation of MultTaper Spectrogram
    tfrz = mean_hmt5(MSp,opt1) ; % means over tapers
end;

if figdisplay,
    figure(3); clf;
    imageTF(tfrz(1:Nfft/2,:),dyn,1,0);
    axis('square');
    title('TFR of the current Surrogate');
end;

%---- STATISTICS for the test --------------------------------------

tfr=tfrz(1:Nfft/2,:);
[theta0(jj),Cn_distz,Cn_meanz]=statio_test_theta(tfr,ttred,opt_dist);

if figdisplay,
    figure(4); plot((Cn_distz-Cn_meanz).^2,'b');
end;

end ;   % for jj


%% COMPUTATION of REFERENCE STATISTICS: Gamma Modeling %%%%%%%%%%%

%%%% theta0:sqrt(theta0);
%%%% theta1=sqrt(theta1);

% Nhist=20;
gam_hat = gamfit(theta0,'mle') ;

% Verification of the Gamma fit
[hh,bb] = hist(theta0,Nhist) ;
hh_est = gampdf(bb,gam_hat(1),gam_hat(2)) ;
dbb = bb(2)-bb(1) ; 

% if figdisplay,
%     figure(21); clf;
%     bar(bb,hh/(JJ)/dbb)
%     hold on
%     plot(bb,hh_est,'r','LineWidth',2);
%     lesaxes=axis;   
%     plot([theta1 theta1],[lesaxes(3) lesaxes(4)],'m','LineWidth',2);
%     hold off; 
% end;


%% TEST for STATIONARITY %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%----- Fix threshold ---------------------
% fa_rate = 0.05 ;
gam_thresh=gaminv(1-fa_rate,gam_hat(1),gam_hat(2)) ;

%----- Compare to Theta1 -----------------

res_test = (theta1>gam_thresh) ;

%----- Compute INS ---------------------

INS = sqrt(theta1/mean(theta0)) ;
%%%% INS = (theta1/mean(theta0)) ;

%----- Convert gam_thres in threshold in the INS space -----

INS_thresh=sqrt(gam_thresh/(gam_hat(1)*gam_hat(2))) ;
%%%% INS_thresh=(gam_thresh/(gam_hat(1)*gam_hat(2))) ;


%% Display results

if resdisplay,
    figure(5); clf;
    bar(bb,hh/(JJ)/dbb)
    hold on
    plot(bb,hh_est,'r','LineWidth',2);
    lesaxes=axis;   
    plot([theta1 theta1],[lesaxes(3) lesaxes(4)],'m','LineWidth',2);
    plot([gam_thresh gam_thresh],[lesaxes(3) lesaxes(4)],'k--','LineWidth',2);
    hold off; 
    title(['Result test = ' num2str(res_test) ' | INS = ' num2str(INS)]);

    figure(2); clf; imageTF(tfrx(1:Nfft/2,:),dyn,1,0); axis('square'); title('Original Signal TFR');
    figure(3); clf; imageTF(tfrz(1:Nfft/2,:),dyn,1,0); axis('square'); title('Example of Surrogate TFR')
figure(5)
end;


%figure(21);

%% SAVE results in OUTPUT structure

% Todo : think about an intelligent way for that...



